@@ -5,21 +5,41 @@ import logging
 import psycopg2
 import itertools
 import psycopg2.extras
-from decimal import Decimal
 from psycopg2.extensions import adapt
+from psycopg2.extras import HstoreAdapter
 
-__version__ = VERSION = version = '0.2.1'
+# http://initd.org/psycopg/docs/module.html#exceptions
+from psycopg2 import Warning
+from psycopg2 import Error
+from psycopg2 import InterfaceError
+from psycopg2 import DatabaseError
+from psycopg2 import DataError
+from psycopg2 import OperationalError
+from psycopg2 import IntegrityError
+from psycopg2 import InternalError
+from psycopg2 import ProgrammingError
+from psycopg2 import NotSupportedError
+
+
+__version__ = VERSION = version = '1.0.0'
 
 from .pubsub import PubSub
 
 
 class Connection(object):
-    def __init__(self, host_or_url="127.0.0.1", database=None, user=None, password=None, port=5432, search_path=None, timezone="+00"):
+    def __init__(self, host_or_url=None, database=None, user=None, password=None, port=5432, 
+                       search_path=None, timezone="+00"):
         self.logging = False
+        if host_or_url is None:
+            host_or_url = os.getenv('TORNPSQL', "127.0.0.1")
         if host_or_url.startswith('postgres://'):
-            args = re.search('postgres://(?P<user>[\w\-]*):?(?P<password>[\w\-]*)@(?P<host>[\w\-\.]+):?(?P<port>\d+)/?(?P<database>[\w\-]+)', host_or_url).groupdict()
-            self.host = args.get('host')
-            self.database = args.get('database')
+            try:
+                args = re.search('postgres://(?P<user>[\w\-]*):?(?P<password>[\w\-]*)@(?P<host>[\w\-\.]+):?(?P<port>\d+)/?(?P<database>[\w\-]+)', host_or_url).groupdict()
+            except:
+                raise ValueError("PostgreSQL url is not a valid format postgres://user:password@host:post/database from %s" % host_or_url)
+            else:
+                self.host = args.get('host')
+                self.database = args.get('database')
         else:
             self.host = host_or_url
             self.database = database
@@ -30,14 +50,20 @@ class Connection(object):
         self._db = None
         self._db_args = args
         self._register_types = []
-        self._search_path = search_path
+        self._search_path = search_path or "public"
         self._change_path = None
         try:
             self.reconnect()
-        except Exception:
+        except Exception: # pragma: no cover
             logging.error("Cannot connect to PostgreSQL on postgresql://%s:<password>@%s/%s", 
                 args['user'], self.host, self.database, exc_info=True)
 
+        psycopg2.extras.register_hstore(self._db, globally=True)
+        psycopg2.extensions.register_adapter(dict, psycopg2.extras.Json)
+
+    def hstore(self, _dict):
+        return HstoreAdapter(_dict)
+
     def __del__(self):
         self.close()
 
@@ -53,18 +79,10 @@ class Connection(object):
         self._db = psycopg2.connect(**self._db_args)
         self._db.autocommit = True
 
-        # register money type
-        psycopg2.extensions.register_type(psycopg2.extensions.new_type((790,), "MONEY", self._cast_money))
-
         # register custom types
         for _type in self._register_types:
             psycopg2.extensions.register_type(psycopg2.extensions.new_type(*_type))
 
-        try:
-            psycopg2.extras.register_hstore(self._db, globally=True)
-        except psycopg2.ProgrammingError:
-            pass
-
     def path(self, search_path):
         self._change_path = search_path
         return self
@@ -72,14 +90,6 @@ class Connection(object):
     def adapt(self, value):
         return adapt(value)
 
-    def hstore(self, dict):
-        return ','.join(['"%s"=>"%s"' % (str(k), str(v)) for k, v in dict.items()])
-
-    def _cast_money(self, s, cur):
-        if s is None:
-            return None
-        return Decimal(s.replace(",","").replace("$",""))
-
     def register_type(self, oids, name, casting):
         """Callback to register data types when reconnect
         """
@@ -87,10 +97,9 @@ class Connection(object):
         assert type(name) in (unicode, str)
         assert hasattr(casting, "__call__")
         self._register_types.append((oids, name, casting))
-        if self._db is not None:
-            psycopg2.extensions.register_type(psycopg2.extensions.new_type(oids, name, casting))
+        psycopg2.extensions.register_type(psycopg2.extensions.new_type(oids, name, casting))
 
-    def mogrify(self, query, *parameters):
+    def mogrify(self, query, *parameters, **kwargs):
         """From http://initd.org/psycopg/docs/cursor.html?highlight=mogrify#cursor.mogrify
         Return a query string after arguments binding.
         The string returned is exactly the one that would be sent to the database running 
@@ -98,8 +107,10 @@ class Connection(object):
         """
         cursor = self._cursor()
         try:
+            if kwargs:
+                return cursor.mogrify(query % dict(map(lambda r: (r[0], adapt(r[1])), kwargs.items())))
             return cursor.mogrify(query, parameters)
-        except:
+        except: # pragma: no cover
             cursor.close()
             raise
 
@@ -125,7 +136,7 @@ class Connection(object):
         if not rows:
             return None
         elif len(rows) > 1:
-            raise Exception("Multiple rows returned for Database.get() query")
+            raise ValueError("Multiple rows returned for get() query")
         else:
             return rows[0]
 
@@ -135,20 +146,10 @@ class Connection(object):
         cursor = self._cursor()
         try:
             self._executemany(cursor, query, parameters)
-            return True
-        except Exception:
+        except Exception: # pragma: no cover
             cursor.close()
             raise
 
-    def execute_rowcount(self, query, *parameters, **kwargs):
-        """Executes the given query, returning the rowcount from the query."""
-        cursor = self._cursor()
-        try:
-            self._execute(cursor, query, parameters, kwargs)
-            return cursor.rowcount
-        finally:
-            cursor.close()
-
     def _ensure_connected(self):
         if self._db is None:
             self.reconnect()
@@ -171,40 +172,29 @@ class Connection(object):
         try:
             query = self._set_search_path(query)
             if kwargs:
-                datas = []
-                keys = []
-                values = []
-                args = []
-                for key in kwargs:
-                    keys.append(key) # for insert
-                    values.append("%s") # for insert
-                    datas.append(key+"=%s") # for update
-                    args.append(kwargs[key])
-                parampos = min([x for x, part in enumerate(query.split("%s")) if part.find('__data__') > -1] or [0])
-                args.reverse()
-                parameters = list(parameters)
-                [parameters.insert(parampos, value) for value in args]
-                query = query.replace("__data__", ','.join(datas))
-                query = query.replace("__keys__", ','.join(keys))
-                query = query.replace("__values__", ','.join(values))
-                parameters = tuple(parameters)
-                
-            if self.logging:
-                logging.info(re.sub(r"\n\s*", " ", cursor.mogrify(query, parameters)))
-            cursor.execute(query, parameters)
-        except psycopg2.OperationalError as e:
+                query = query % dict(map(lambda r: (r[0], adapt(r[1])), kwargs.items()))
+                if self.logging: self._log(query)
+                cursor.execute(query)
+            else:
+                if self.logging: self._log(cursor.mogrify(query, parameters))
+                cursor.execute(query, parameters)
+
+        except psycopg2.OperationalError as e: # pragma: no cover
             logging.error("Error connecting to PostgreSQL on %s, %s", self.host, e)
             self.close()
             raise
 
+    def _log(self, query):    
+        logging.info(re.sub(r"\n\s*", " ", query))
+
     def _executemany(self, cursor, query, parameters):
-        """The function is mostly useful for commands that update the database: any result set returned by the query is discarded."""
+        """The function is mostly useful for commands that update the database: 
+           any result set returned by the query is discarded."""
         try:
             query = self._set_search_path(query)
-            if self.logging:
-                logging.info(re.sub(r"\n\s*", " ", cursor.mogrify(query, parameters)))
+            if self.logging: self._log(query)
             cursor.executemany(query, parameters)
-        except psycopg2.OperationalError as e:
+        except psycopg2.OperationalError as e: # pragma: no cover
             logging.error("Error connecting to PostgreSQL on %s, e", self.host, e)
             self.close()
             raise 
@@ -232,14 +222,12 @@ class Connection(object):
         """pops and returns all notices
         http://initd.org/psycopg/docs/connection.html#connection.notices
         """
-        if self._db:
-            return [self._db.notices.pop()[8:].strip() for x in range(len(self._db.notices))]
-        return []
+        return [self._db.notices.pop()[8:].strip() for x in range(len(self._db.notices))]
 
 class Row(dict):
     """A dict that allows for object-like property access syntax."""
     def __getattr__(self, name):
         try:
             return self[name]
-        except KeyError:
+        except KeyError:  # pragma: no cover
             raise AttributeError(name)
